# -*- coding: utf-8 -*-
#
# Copyright (C) 2006 Christopher Lenz <cmlenz@gmx.de>
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://trac.edgewall.com/license.html.

from trac.core import *
from trac.mimeview.api import IHTMLPreviewRenderer
from trac.util import escape

__all__ = ['ExcelRenderer']

try:
    import xlrd
    have_xrld = True
except ImportError:
    have_xrld = False

    
def get_color(book, color, styleName) :
    #Trying default value
    if color == None:
        color = book.colour_map[0x7FFF]
    colorStyle=''
    if not color == None:
        colorStyle='%s:rgb(%d,%d,%d)' % (styleName,color[0],color[1],color[2])
    return colorStyle

class ExcelRenderer(Component):
    """Syntax highlighting using GNU Enscript."""

    implements(IHTMLPreviewRenderer)

    MIME_TYPES = ('application/vnd.ms-excel', 'application/excel')

    # IHTMLPreviewRenderer methods

    def get_quality_ratio(self, mimetype):
        if have_xrld and mimetype in self.MIME_TYPES:
            return 2
        return 0

    def render(self, req, mimetype, content, filename=None, url=None):
        if hasattr(content, 'read'):
            content = content.read()
        book = xlrd.open_workbook(file_contents=content, formatting_info=True)
        buf = []
        for sheet_name in book.sheet_names():
            sheet = book.sheet_by_name(sheet_name)
            if not sheet.nrows:
                continue # skip empty sheets
            merged ={}
            skipped ={}
	    for crange in sheet.merged_cells:
	        rlo, rhi, clo, chi = crange
	        merged[(rlo,clo)]=(rhi-rlo,chi-clo) #colspan,rowspan
	        for rx in xrange(rlo, rhi):
		    for cx in xrange(clo, chi):
		        skipped[(rx,cx)] = True
		skipped[(rlo,clo)] = False
            buf.append(u'<table class="listing"><caption>%s</caption>\n' %
                       escape(sheet.name))
            buf.append(u'<tbody>')
            for ridx in range(sheet.nrows):
                row = sheet.row(ridx)
                if not filter(lambda cell: unicode(cell.value), row):
                    continue
                buf.append(u'<tr class="%s" id="row%d">' % (ridx % 2 and 'odd' or 'even', ridx + 1))
                for cidx, cell in enumerate(row):
                    key = (ridx,cidx)
                    if key in skipped and skipped[key]:
                        continue #skipping - merged columns
                    formatIndex = sheet.cell_xf_index(ridx, cidx)
		    format = book.computed_xf_list[formatIndex]
		    font = book.font_list[format.font_index]
		    color = book.colour_map[font.colour_index]
		    bgColor = book.colour_map[format.background.pattern_colour_index]
		    style = 'normal'
		    if font.italic:
			style='italic'
		    decor = 'none'
		    if font.underlined:
			decor='underline'
		    colspan=''
		    rowspan=''
		    if key in merged:
			spans = merged[key]
			rowspan='rowspan="%d"' % spans[0]
			colspan='colspan="%d"' % spans[1]
		    buf.append(u'<td style="font-weight:%(weight)d; font-style:%(style)s; %(colorStyle)s; %(bgColorStyle)s; text-decoration:%(decor)s " %(rowspan)s %(colspan)s>%(value)s</td>' %
			{
			"weight":font.weight,
			"style":style,
			"value":sheet.cell_value(ridx, cidx),
			"colorStyle":get_color(book,color,'color'),
			"bgColorStyle":get_color(book,bgColor,'background-color'),
			"decor":decor,
			"colspan":colspan,
			"rowspan":rowspan
			})
                buf.append(u'</tr>\n')
            buf.append(u'</tbody>')
            buf.append(u'</table>\n')
        return u''.join(buf)
